@@ -12,10 +12,6 @@
 #'
 #' Argument to check.
 #'
-#' @param ...
-#'
-#' Additional fields that will be added to the resulting error condition
-#'
 #' @param .arg `[character]`
 #'
 #' Name of argument being checked (used in error message).
@@ -28,10 +24,29 @@
 #'
 #' The (optional) subclass of the returned error condition.
 #'
+#' @param lower `[numeric]`
+#'
+#' The lower bound to compare against.
+#'
+#' @param upper `[numeric]`
+#'
+#' The upper bound to compare against.
+#'
+#' @param left_inclusive `[bool]`
+#'
+#' Should the left (lower) bound be compared inclusively (`<=`) or exclusive (`<`).
+#'
+#' @param right_inclusive `[bool]`
+#'
+#' Should the right (upper) bound be compared inclusively (`>=`) or exclusive (`>`).
+#'
 # -------------------------------------------------------------------------
 #' @return
 #'
-#' NULL if the assertion succeeds (error otherwise).
+#' NULL if the assertion succeeds.
+#'
+#' Otherwise an error of class "ympes-error" (with optional subclass if supplied
+#' when calling the assertion).
 #'
 # -------------------------------------------------------------------------
 #' @examples
@@ -42,7 +57,6 @@
 #'     TRUE
 #' }
 #' fun(i=1L)
-#' try(fun())
 #' try(fun(i="cat"))
 #'
 #' # Use in an internal function
@@ -59,7 +73,6 @@
 #'     internal_fun(a=b)
 #' }
 #' external_fun(b="cat")
-#' try(external_fun())
 #' try(external_fun(b = letters))
 #' tryCatch(external_fun(b = letters), error = class)
 #'
@@ -72,15 +85,17 @@ NULL
 #' @export
 assert_integer <- function(
     x,
-    ...,
     .arg = deparse(substitute(x)),
     .call = sys.call(-1L),
     .subclass = NULL
 ) {
-    .assert_not_missing(x, ..., .arg = .arg, .call = .call, .subclass = .subclass)
     if (!is.integer(x)) {
-        msg <- sprintf("`%s` must be an integer vector.", .arg)
-        .stop(msg, ..., .call = .call, .subclass = .subclass)
+        .stopf(
+            gettextf("`%s` must be an integer vector.", .arg, domain = "R-ympes"),
+            .call = .call,
+            .subclass = .subclass
+
+        )
     }
 }
 
@@ -89,21 +104,45 @@ assert_integer <- function(
 #' @export
 assert_int <- assert_integer
 
+# -------------------------------------------------------------------------
+#' @rdname assertions
+#' @export
+assert_integer_not_na <- function(
+    x,
+    .arg = deparse(substitute(x)),
+    .call = sys.call(-1L),
+    .subclass = NULL
+) {
+    if (!is.integer(x) || anyNA(x)) {
+        .stopf(
+            gettextf("`%s` must be a non-missing integer vector.", .arg, domain = "R-ympes"),
+            .call = .call,
+            .subclass = .subclass
+
+        )
+    }
+}
+
+# -------------------------------------------------------------------------
+#' @rdname assertions
+#' @export
+assert_int_not_na <- assert_integer_not_na
 
 # -------------------------------------------------------------------------
 #' @rdname assertions
 #' @export
 assert_double <- function(
     x,
-    ...,
     .arg = deparse(substitute(x)),
     .call = sys.call(-1L),
     .subclass = NULL
 ) {
-    .assert_not_missing(x, ..., .arg = .arg, .call = .call, .subclass = .subclass)
     if (!is.double(x)) {
-        msg <- sprintf("`%s` must be a double vector.", .arg)
-        .stop(msg, ..., .call = .call, .subclass = .subclass)
+        .stopf(
+            gettextf("`%s` must be a double vector.", .arg, domain = "R-ympes"),
+            .call = .call,
+            .subclass = .subclass
+        )
     }
 }
 
@@ -115,17 +154,41 @@ assert_dbl <- assert_double
 # -------------------------------------------------------------------------
 #' @rdname assertions
 #' @export
+assert_double_not_na <- function(
+    x,
+    .arg = deparse(substitute(x)),
+    .call = sys.call(-1L),
+    .subclass = NULL
+) {
+    if (!is.double(x) || anyNA(x)) {
+        .stopf(
+            gettextf("`%s` must be a non-missing double vector.", .arg, domain = "R-ympes"),
+            .call = .call,
+            .subclass = .subclass
+        )
+    }
+}
+
+# -------------------------------------------------------------------------
+#' @rdname assertions
+#' @export
+assert_dbl_not_na <- assert_double_not_na
+
+# -------------------------------------------------------------------------
+#' @rdname assertions
+#' @export
 assert_numeric <- function(
     x,
-    ...,
     .arg = deparse(substitute(x)),
     .call = sys.call(-1L),
     .subclass = NULL
 ) {
-    .assert_not_missing(x, ..., .arg = .arg, .call = .call, .subclass = .subclass)
     if (!is.numeric(x)) {
-        msg <- sprintf("`%s` must be a numeric vector.", .arg)
-        .stop(msg, ..., .call = .call, .subclass = .subclass)
+        .stopf(
+            gettextf("`%s` must be a numeric vector.", .arg, domain = "R-ympes"),
+            .call = .call,
+            .subclass = .subclass
+        )
     }
 }
 
@@ -137,17 +200,46 @@ assert_num <- assert_numeric
 # -------------------------------------------------------------------------
 #' @rdname assertions
 #' @export
+assert_dbl_not_na <- assert_double_not_na
+
+# -------------------------------------------------------------------------
+#' @rdname assertions
+#' @export
+assert_numeric_not_na <- function(
+    x,
+    .arg = deparse(substitute(x)),
+    .call = sys.call(-1L),
+    .subclass = NULL
+) {
+    if (!is.numeric(x) || anyNA(x)) {
+        .stopf(
+            gettextf("`%s` must be a non-missing numeric vector.", .arg, domain = "R-ympes"),
+            .call = .call,
+            .subclass = .subclass
+        )
+    }
+}
+
+# -------------------------------------------------------------------------
+#' @rdname assertions
+#' @export
+assert_num_not_na <- assert_numeric_not_na
+
+# -------------------------------------------------------------------------
+#' @rdname assertions
+#' @export
 assert_logical <- function(
     x,
-    ...,
     .arg = deparse(substitute(x)),
     .call = sys.call(-1L),
     .subclass = NULL
 ) {
-    .assert_not_missing(x, ..., .arg = .arg, .call = .call, .subclass = .subclass)
     if (!is.logical(x)) {
-        msg <- sprintf("`%s` must be a logical vector.", .arg)
-        .stop(msg, ..., .call = .call, .subclass = .subclass)
+        .stopf(
+            gettextf("`%s` must be a logical vector.", .arg, domain = "R-ympes"),
+            .call = .call,
+            .subclass = .subclass
+        )
     }
 }
 
@@ -159,17 +251,41 @@ assert_lgl <- assert_logical
 # -------------------------------------------------------------------------
 #' @rdname assertions
 #' @export
+assert_logical_not_na <- function(
+    x,
+    .arg = deparse(substitute(x)),
+    .call = sys.call(-1L),
+    .subclass = NULL
+) {
+    if (!is.logical(x) || anyNA(x)) {
+        .stopf(
+            gettextf("`%s` must be a non-missing logical vector.", .arg, domain = "R-ympes"),
+            .call = .call,
+            .subclass = .subclass
+        )
+    }
+}
+
+# -------------------------------------------------------------------------
+#' @rdname assertions
+#' @export
+assert_lgl_not_na <- assert_logical_not_na
+
+# -------------------------------------------------------------------------
+#' @rdname assertions
+#' @export
 assert_character <- function(
     x,
-    ...,
     .arg = deparse(substitute(x)),
     .call = sys.call(-1L),
     .subclass = NULL
 ) {
-    .assert_not_missing(x, ..., .arg = .arg, .call = .call, .subclass = .subclass)
     if (!is.character(x)) {
-        msg <- sprintf("`%s` must be a character vector.", .arg)
-        .stop(msg, ..., .call = .call, .subclass = .subclass)
+        .stopf(
+            gettextf("`%s` must be a character vector.", .arg, domain = "R-ympes"),
+            .call = .call,
+            .subclass = .subclass
+        )
     }
 }
 
@@ -181,17 +297,41 @@ assert_chr <- assert_character
 # -------------------------------------------------------------------------
 #' @rdname assertions
 #' @export
+assert_character_not_na <- function(
+    x,
+    .arg = deparse(substitute(x)),
+    .call = sys.call(-1L),
+    .subclass = NULL
+) {
+    if (!is.character(x) || anyNA(x)) {
+        .stopf(
+            gettextf("`%s` must be a non-missing character vector.", .arg, domain = "R-ympes"),
+            .call = .call,
+            .subclass = .subclass
+        )
+    }
+}
+
+# -------------------------------------------------------------------------
+#' @rdname assertions
+#' @export
+assert_chr_not_na <- assert_character_not_na
+
+# -------------------------------------------------------------------------
+#' @rdname assertions
+#' @export
 assert_data_frame <- function(
     x,
-    ...,
     .arg = deparse(substitute(x)),
     .call = sys.call(-1L),
     .subclass = NULL
 ) {
-    .assert_not_missing(x, ..., .arg = .arg, .call = .call, .subclass = .subclass)
     if (!is.data.frame(x)) {
-        msg <- sprintf("`%s` must be a data frame.", .arg)
-        .stop(msg, ..., .call = .call, .subclass = .subclass)
+        .stopf(
+            gettextf("`%s` must be a data frame.", .arg, domain = "R-ympes"),
+            .call = .call,
+            .subclass = .subclass
+        )
     }
 }
 
@@ -200,32 +340,58 @@ assert_data_frame <- function(
 #' @export
 assert_list <- function(
     x,
-    ...,
     .arg = deparse(substitute(x)),
     .call = sys.call(-1L),
     .subclass = NULL
 ) {
-    .assert_not_missing(x, ..., .arg = .arg, .call = .call, .subclass = .subclass)
     if (!is.list(x)) {
-        msg <- sprintf("`%s` must be a list.", .arg)
-        .stop(msg, ..., .call = .call, .subclass = .subclass)
+        .stopf(
+            gettextf("`%s` must be a list.", .arg, domain = "R-ympes"),
+            .call = .call,
+            .subclass = .subclass
+        )
     }
 }
 
 # -------------------------------------------------------------------------
 #' @rdname assertions
 #' @export
+assert_whole <- function(
+    x,
+    .arg = deparse(substitute(x)),
+    .call = sys.call(-1L),
+    .subclass = NULL
+) {
+    if (!.is_whole(x)) {
+        .stopf(
+            gettextf("`%s` must be integerish.", .arg, domain = "R-ympes"),
+            .call = .call,
+            .subclass = .subclass
+        )
+    }
+}
+
+# -------------------------------------------------------------------------
+#' @rdname assertions
+#' @export
+assert_integerish <- assert_whole
+
+
+# -------------------------------------------------------------------------
+#' @rdname assertions
+#' @export
 assert_scalar_integer <- function(
     x,
-    ...,
     .arg = deparse(substitute(x)),
     .call = sys.call(-1L),
     .subclass = NULL
 ) {
-    .assert_not_missing(x, ..., .arg = .arg, .call = .call, .subclass = .subclass)
     if (!(is.integer(x) && length(x) == 1L)) {
-        msg <- sprintf("`%s` must be an integer vector of length 1.", .arg)
-        .stop(msg, ..., .call = .call, .subclass = .subclass)
+        .stopf(
+            gettextf("`%s` must be an integer vector of length 1.", .arg, domain = "R-ympes"),
+            .call = .call,
+            .subclass = .subclass
+        )
     }
 }
 
@@ -239,15 +405,16 @@ assert_scalar_int <- assert_scalar_integer
 #' @export
 assert_scalar_integer_not_na <- function(
     x,
-    ...,
     .arg = deparse(substitute(x)),
     .call = sys.call(-1L),
     .subclass = NULL
 ) {
-    .assert_not_missing(x, ..., .arg = .arg, .call = .call, .subclass = .subclass)
-    if (!(is.integer(x) && length(x) == 1L) || is.na(x)) {
-        msg <- sprintf("`%s` must be an integer vector of length 1 and not NA.", .arg)
-        .stop(msg, ..., .call = .call, .subclass = .subclass)
+    if (!(is.integer(x) && length(x) == 1L) || anyNA(x)) {
+        .stopf(
+            gettextf("`%s` must be an integer vector of length 1 and not NA.", .arg, domain = "R-ympes"),
+            .call = .call,
+            .subclass = .subclass
+        )
     }
 }
 
@@ -261,15 +428,16 @@ assert_scalar_int_not_na <- assert_scalar_integer_not_na
 #' @export
 assert_scalar_double <- function(
     x,
-    ...,
     .arg = deparse(substitute(x)),
     .call = sys.call(-1L),
     .subclass = NULL
 ) {
-    .assert_not_missing(x, ..., .arg = .arg, .call = .call, .subclass = .subclass)
     if (!(is.double(x) && length(x) == 1L)) {
-        msg <- sprintf("`%s` must be a double vector of length 1.", .arg)
-        .stop(msg, ..., .call = .call, .subclass = .subclass)
+        .stopf(
+            gettextf("`%s` must be a double vector of length 1.", .arg, domain = "R-ympes"),
+            .call = .call,
+            .subclass = .subclass
+        )
     }
 }
 
@@ -283,15 +451,16 @@ assert_scalar_dbl <- assert_scalar_double
 #' @export
 assert_scalar_double_not_na <- function(
     x,
-    ...,
     .arg = deparse(substitute(x)),
     .call = sys.call(-1L),
     .subclass = NULL
 ) {
-    .assert_not_missing(x, ..., .arg = .arg, .call = .call, .subclass = .subclass)
-    if (!(is.double(x) && length(x) == 1L) || is.na(x)) {
-        msg <- sprintf("`%s` must be a double vector of length 1 and not NA.", .arg)
-        .stop(msg, ..., .call = .call, .subclass = .subclass)
+    if (!(is.double(x) && length(x) == 1L) || anyNA(x)) {
+        .stopf(
+            gettextf("`%s` must be a double vector of length 1 and not NA.", .arg, domain = "R-ympes"),
+            .call = .call,
+            .subclass = .subclass
+        )
     }
 }
 
@@ -305,15 +474,16 @@ assert_scalar_dbl_not_na <- assert_scalar_double_not_na
 #' @export
 assert_scalar_numeric <- function(
     x,
-    ...,
     .arg = deparse(substitute(x)),
     .call = sys.call(-1L),
     .subclass = NULL
 ) {
-    .assert_not_missing(x, ..., .arg = .arg, .call = .call, .subclass = .subclass)
     if (!(is.numeric(x) && length(x) == 1L)) {
-        msg <- sprintf("`%s` must be a numeric vector of length 1.", .arg)
-        .stop(msg, ..., .call = .call, .subclass = .subclass)
+        .stopf(
+            gettextf("`%s` must be a numeric vector of length 1.", .arg, domain = "R-ympes"),
+            .call = .call,
+            .subclass = .subclass
+        )
     }
 }
 
@@ -327,15 +497,16 @@ assert_scalar_num <- assert_scalar_numeric
 #' @export
 assert_scalar_numeric_not_na <- function(
     x,
-    ...,
     .arg = deparse(substitute(x)),
     .call = sys.call(-1L),
     .subclass = NULL
 ) {
-    .assert_not_missing(x, ..., .arg = .arg, .call = .call, .subclass = .subclass)
-    if (!(is.numeric(x) && length(x) == 1L) || is.na(x)) {
-        msg <- sprintf("`%s` must be a numeric vector of length 1 and not NA.", .arg)
-        .stop(msg, ..., .call = .call, .subclass = .subclass)
+    if (!(is.numeric(x) && length(x) == 1L) || anyNA(x)) {
+        .stopf(
+            gettextf("`%s` must be a numeric vector of length 1 and not NA.", .arg, domain = "R-ympes"),
+            .call = .call,
+            .subclass = .subclass
+        )
     }
 }
 
@@ -349,15 +520,16 @@ assert_scalar_num_not_na <- assert_scalar_numeric_not_na
 #' @export
 assert_scalar_logical <- function(
     x,
-    ...,
     .arg = deparse(substitute(x)),
     .call = sys.call(-1L),
     .subclass = NULL
 ) {
-    .assert_not_missing(x, ..., .arg = .arg, .call = .call, .subclass = .subclass)
     if (!(is.logical(x) && length(x) == 1L)) {
-        msg <- sprintf("`%s` must be a logical vector of length 1.", .arg)
-        .stop(msg, ..., .call = .call, .subclass = .subclass)
+        .stopf(
+            gettextf("`%s` must be a logical vector of length 1.", .arg, domain = "R-ympes"),
+            .call = .call,
+            .subclass = .subclass
+        )
     }
 }
 
@@ -369,34 +541,64 @@ assert_scalar_lgl <- assert_scalar_logical
 # -------------------------------------------------------------------------
 #' @rdname assertions
 #' @export
-assert_scalar_whole <- function(
+assert_scalar_logical_not_na <- function(
     x,
-    ...,
     .arg = deparse(substitute(x)),
     .call = sys.call(-1L),
     .subclass = NULL
 ) {
-    .assert_not_missing(x, ..., .arg = .arg, .call = .call, .subclass = .subclass)
-    if (!.is_scalar_whole(x)) {
-        msg <- sprintf("`%s` must be integerish and of length 1.", .arg)
-        .stop(msg, ..., .call = .call, .subclass = .subclass)
+    if (!(is.logical(x) && length(x) == 1L) || anyNA(x)) {
+        .stopf(
+            gettextf("`%s` must be a logical vector of length 1 and not NA.", .arg, domain = "R-ympes"),
+            .call = .call,
+            .subclass = .subclass
+        )
     }
 }
 
+# -------------------------------------------------------------------------
+#' @rdname assertions
+#' @export
+assert_scalar_lgl_not_na <- assert_scalar_logical_not_na
+
+# -------------------------------------------------------------------------
+#' @rdname assertions
+#' @export
+assert_scalar_whole <- function(
+    x,
+    .arg = deparse(substitute(x)),
+    .call = sys.call(-1L),
+    .subclass = NULL
+) {
+    if (!.is_scalar_whole(x)) {
+        .stopf(
+            gettextf("`%s` must be integerish and of length 1.", .arg, domain = "R-ympes"),
+            .call = .call,
+            .subclass = .subclass
+        )
+    }
+}
+
+# -------------------------------------------------------------------------
+#' @rdname assertions
+#' @export
+assert_scalar_integerish <- assert_scalar_whole
+
 
 #' @rdname assertions
 #' @export
 assert_bool <- function(
     x,
-    ...,
     .arg = deparse(substitute(x)),
     .call = sys.call(-1L),
     .subclass = NULL
 ) {
-    .assert_not_missing(x, ..., .arg = .arg, .call = .call, .subclass = .subclass)
-    if (!(is.logical(x) && length(x) == 1L) || is.na(x)) {
-        msg <- sprintf("`%s` must be boolean (TRUE/FALSE).", .arg)
-        .stop(msg, ..., .call = .call, .subclass = .subclass)
+    if (!(is.logical(x) && length(x) == 1L) || anyNA(x)) {
+        .stopf(
+            gettextf("`%s` must be boolean (TRUE/FALSE).", .arg, domain = "R-ympes"),
+            .call = .call,
+            .subclass = .subclass
+        )
     }
 }
 
@@ -410,15 +612,16 @@ assert_boolean <- assert_bool
 #' @export
 assert_scalar_character <- function(
     x,
-    ...,
     .arg = deparse(substitute(x)),
     .call = sys.call(-1L),
     .subclass = NULL
 ) {
-    .assert_not_missing(x, ..., .arg = .arg, .call = .call, .subclass = .subclass)
     if (!(is.character(x) && length(x) == 1L)) {
-        msg <- sprintf("`%s` must be a character vector of length 1.", .arg)
-        .stop(msg, ..., .call = .call, .subclass = .subclass)
+        .stopf(
+            gettextf("`%s` must be a character vector of length 1.", .arg, domain = "R-ympes"),
+            .call = .call,
+            .subclass = .subclass
+        )
     }
 }
 
@@ -427,20 +630,26 @@ assert_scalar_character <- function(
 #' @export
 assert_scalar_chr <- assert_scalar_character
 
+# -------------------------------------------------------------------------
+#' @rdname assertions
+#' @export
+assert_string <- assert_scalar_character
+
 # -------------------------------------------------------------------------
 #' @rdname assertions
 #' @export
 assert_scalar_character_not_na <- function(
     x,
-    ...,
     .arg = deparse(substitute(x)),
     .call = sys.call(-1L),
     .subclass = NULL
 ) {
-    .assert_not_missing(x, ..., .arg = .arg, .call = .call, .subclass = .subclass)
-    if (!(is.character(x) && length(x) == 1L) || is.na(x)) {
-        msg <- sprintf("`%s` must be a character vector of length 1 and not NA.", .arg)
-        .stop(msg, ..., .call = .call, .subclass = .subclass)
+    if (!(is.character(x) && length(x) == 1L) || anyNA(x)) {
+        .stopf(
+            gettextf("`%s` must be a character vector of length 1 and not NA.", .arg, domain = "R-ympes"),
+            .call = .call,
+            .subclass = .subclass
+        )
     }
 }
 
@@ -452,28 +661,24 @@ assert_scalar_chr_not_na <- assert_scalar_character_not_na
 # -------------------------------------------------------------------------
 #' @rdname assertions
 #' @export
-assert_string <- assert_scalar_chr
+assert_string_not_na <- assert_scalar_character_not_na
 
 # -------------------------------------------------------------------------
 #' @rdname assertions
 #' @export
 assert_non_negative_or_na <- function(
     x,
-    ...,
     .arg = deparse(substitute(x)),
     .call = sys.call(-1L),
     .subclass = NULL
 ) {
-    .assert_not_missing(x, ..., .arg = .arg, .call = .call, .subclass = .subclass)
-
-    if (!is.numeric(x)) {
-        msg <- sprintf("`%s` must be a numeric vector.", .arg)
-        .stop(msg, ..., .call = .call, .subclass = .subclass)
-    }
-
+    assert_numeric(x, .arg = .arg, .call = .call, .subclass = .subclass)
     if (!.all_non_negative_or_na(x)) {
-        msg <- sprintf("`%s` values must be non-negative or NA.", .arg)
-        .stop(msg, ..., .call = .call, .subclass = .subclass)
+        .stopf(
+            gettextf("`%s` values must be non-negative or NA.", .arg, domain = "R-ympes"),
+            .call = .call,
+            .subclass = .subclass
+        )
     }
 }
 
@@ -482,21 +687,17 @@ assert_non_negative_or_na <- function(
 #' @export
 assert_non_positive_or_na <- function(
     x,
-    ...,
     .arg = deparse(substitute(x)),
     .call = sys.call(-1L),
     .subclass = NULL
 ) {
-    .assert_not_missing(x, ..., .arg = .arg, .call = .call, .subclass = .subclass)
-
-    if (!is.numeric(x)) {
-        msg <- sprintf("`%s` must be a numeric vector.", .arg)
-        .stop(msg, ..., .call = .call, .subclass = .subclass)
-    }
-
+    assert_numeric(x, .arg = .arg, .call = .call, .subclass = .subclass)
     if (!.all_non_positive_or_na(x)) {
-        msg <- sprintf("`%s` values must be non-positive or NA.", .arg)
-        .stop(msg, ..., .call = .call, .subclass = .subclass)
+        .stopf(
+            gettextf("`%s` values must be non-positive or NA.", .arg, domain = "R-ympes"),
+            .call = .call,
+            .subclass = .subclass
+        )
     }
 }
 
@@ -505,21 +706,17 @@ assert_non_positive_or_na <- function(
 #' @export
 assert_non_negative <- function(
     x,
-    ...,
     .arg = deparse(substitute(x)),
     .call = sys.call(-1L),
     .subclass = NULL
 ) {
-    .assert_not_missing(x, ..., .arg = .arg, .call = .call, .subclass = .subclass)
-
-    if (!is.numeric(x)) {
-        msg <- sprintf("`%s` must be a numeric vector.", .arg)
-        .stop(msg, ..., .call = .call, .subclass = .subclass)
-    }
-
+    assert_numeric(x, .arg = .arg, .call = .call, .subclass = .subclass)
     if (!.all_non_negative(x)) {
-        msg <- sprintf("`%s` values must be non-negative and not NA.", .arg)
-        .stop(msg, ..., .call = .call, .subclass = .subclass)
+        .stopf(
+            gettextf("`%s` values must be non-negative and not NA.", .arg, domain = "R-ympes"),
+            .call = .call,
+            .subclass = .subclass
+        )
     }
 
 }
@@ -529,22 +726,17 @@ assert_non_negative <- function(
 #' @export
 assert_non_positive <- function(
     x,
-    ...,
     .arg = deparse(substitute(x)),
     .call = sys.call(-1L),
     .subclass = NULL
 ) {
-    .assert_not_missing(x, ..., .arg = .arg, .call = .call, .subclass = .subclass)
-
-    if (!is.numeric(x)) {
-        msg <- sprintf("`%s` must be a numeric vector.", .arg)
-        .stop(msg, ..., .call = .call, .subclass = .subclass)
-    }
-
-
+    assert_numeric(x, .arg = .arg, .call = .call, .subclass = .subclass)
     if (!.all_non_positive(x)) {
-        msg <- sprintf("`%s` values must be non-positive and not NA.", .arg)
-        .stop(msg, ..., .call = .call, .subclass = .subclass)
+        .stopf(
+            gettextf("`%s` values must be non-positive and not NA.", .arg, domain = "R-ympes"),
+            .call = .call,
+            .subclass = .subclass
+        )
     }
 }
 
@@ -553,21 +745,17 @@ assert_non_positive <- function(
 #' @export
 assert_positive <- function(
     x,
-    ...,
     .arg = deparse(substitute(x)),
     .call = sys.call(-1L),
     .subclass = NULL
 ) {
-    .assert_not_missing(x, ..., .arg = .arg, .call = .call, .subclass = .subclass)
-
-    if (!is.numeric(x)) {
-        msg <- sprintf("`%s` must be a numeric vector.", .arg)
-        .stop(msg, ..., .call = .call, .subclass = .subclass)
-    }
-
+    assert_numeric(x, .arg = .arg, .call = .call, .subclass = .subclass)
     if (!.all_positive(x)) {
-        msg <- sprintf("`%s` values must be positive and not NA.", .arg)
-        .stop(msg, ..., .call = .call, .subclass = .subclass)
+        .stopf(
+            gettextf("`%s` values must be positive and not NA.", .arg, domain = "R-ympes"),
+            .call = .call,
+            .subclass = .subclass
+        )
     }
 }
 
@@ -576,21 +764,17 @@ assert_positive <- function(
 #' @export
 assert_negative <- function(
     x,
-    ...,
     .arg = deparse(substitute(x)),
     .call = sys.call(-1L),
     .subclass = NULL
 ) {
-    .assert_not_missing(x, ..., .arg = .arg, .call = .call, .subclass = .subclass)
-
-    if (!is.numeric(x)) {
-        msg <- sprintf("`%s` must be a numeric vector.", .arg)
-        .stop(msg, ..., .call = .call, .subclass = .subclass)
-    }
-
+    assert_numeric(x, .arg = .arg, .call = .call, .subclass = .subclass)
     if (!.all_negative(x)) {
-        msg <- sprintf("`%s` values must be negative and not NA.", .arg)
-        .stop(msg, ..., .call = .call, .subclass = .subclass)
+        .stopf(
+            gettextf("`%s` values must be negative and not NA.", .arg, domain = "R-ympes"),
+            .call = .call,
+            .subclass = .subclass
+        )
     }
 }
 
@@ -599,21 +783,17 @@ assert_negative <- function(
 #' @export
 assert_positive_or_na <- function(
     x,
-    ...,
     .arg = deparse(substitute(x)),
     .call = sys.call(-1L),
     .subclass = NULL
 ) {
-    .assert_not_missing(x, ..., .arg = .arg, .call = .call, .subclass = .subclass)
-
-    if (!is.numeric(x)) {
-        msg <- sprintf("`%s` must be a numeric vector.", .arg)
-        .stop(msg, ..., .call = .call, .subclass = .subclass)
-    }
-
+    assert_numeric(x, .arg = .arg, .call = .call, .subclass = .subclass)
     if (!.all_positive_or_na(x)) {
-        msg <- sprintf("`%s` values must be positive or NA.", .arg)
-        .stop(msg, ..., .call = .call, .subclass = .subclass)
+        .stopf(
+            gettextf("`%s` values must be positive or NA.", .arg, domain = "R-ympes"),
+            .call = .call,
+            .subclass = .subclass
+        )
     }
 }
 
@@ -622,41 +802,76 @@ assert_positive_or_na <- function(
 #' @export
 assert_negative_or_na <- function(
     x,
-    ...,
     .arg = deparse(substitute(x)),
     .call = sys.call(-1L),
     .subclass = NULL
 ) {
-    .assert_not_missing(x, ..., .arg = .arg, .call = .call, .subclass = .subclass)
-
-    if (!is.numeric(x)) {
-        msg <- sprintf("`%s` must be a numeric vector.", .arg)
-        .stop(msg, ..., .call = .call, .subclass = .subclass)
-    }
-
+    assert_numeric(x, .arg = .arg, .call = .call, .subclass = .subclass)
     if (!.all_negative_or_na(x)) {
-        msg <- sprintf("`%s` values must be negative or NA.", .arg)
-        .stop(msg, ..., .call = .call, .subclass = .subclass)
+        .stopf(
+            gettextf("`%s` values must be negative or NA.", .arg, domain = "R-ympes"),
+            .call = .call,
+            .subclass = .subclass
+        )
+    }
+}
+
+
+# -------------------------------------------------------------------------
+#' @rdname assertions
+#' @export
+assert_between <- function(
+    x,
+    lower = -Inf,
+    upper = Inf,
+    left_inclusive = TRUE,
+    right_inclusive = TRUE,
+    .arg = deparse(substitute(x)),
+    .call = sys.call(-1L),
+    .subclass = NULL
+) {
+
+    assert_numeric_not_na(x, .arg = .arg, .call = .call, .subclass = .subclass)
+    assert_scalar_numeric_not_na(lower, .arg = .arg, .call = .call, .subclass = .subclass)
+    assert_scalar_numeric_not_na(upper, .arg = .arg, .call = .call, .subclass = .subclass)
+    assert_bool(left_inclusive)
+    assert_bool(right_inclusive)
+
+    if (left_inclusive) {
+        left_condition <- lower <= x
+        left_char <- "<="
+    } else {
+        left_condition <- lower < x
+        left_char <- "<"
     }
 
+    if (right_inclusive) {
+        right_condition <- x <= upper
+        right_char <- "<="
+    } else {
+        right_condition <- x < upper
+        right_char <- "<"
+    }
+
+    if (!all(left_condition & right_condition)) {
+        condition_char <- sprintf("lower %s value %s upper", left_char, right_char)
+        .stopf(
+            gettextf("`%s` does not for all values in `%s`.", condition_char, .arg, domain = "R-ympes"),
+            .call = .call,
+            .subclass = .subclass
+        )
+    }
 
 }
 
+
+
 # ------------------------------------------------------------------------- #
 # ------------------------------------------------------------------------- #
 # -------------------------------- INTERNALS ------------------------------ #
 # ------------------------------------------------------------------------- #
 # ------------------------------------------------------------------------- #
 
-.assert_not_missing <- function(x, ..., .arg, .call, .subclass) {
-    if (missing(x)) {
-        msg <- sprintf("argument `%s` is missing, with no default.", .arg)
-        .stop(msg, .call = .call, .subclass = .subclass)
-    }
-}
-
-# -------------------------------------------------------------------------
-
 .all_non_negative_or_na <- function(x) {
     min(0, x, na.rm = TRUE) >= 0
 }
@@ -734,14 +949,32 @@ assert_negative_or_na <- function(
 }
 
 # -------------------------------------------------------------------------
+.is_whole <- function(x, tol = .Machine$double.eps^0.5) {
 
-.is_scalar_whole <- function(x, tol = .Machine$double.eps^0.5) {
-    if (length(x) != 1L || is.na(x))
+    # TODO - document that we do not accept NA_integer_!!!
+    if (anyNA(x))
         return(FALSE)
 
     if (is.integer(x))
         return(TRUE)
 
+    # TODO - note down why I use is.vector here (once I remember)
+    if (is.vector(x, "double") && all(abs(x - round(x)) < tol))
+        return(TRUE)
+
+    FALSE
+}
+
+# -------------------------------------------------------------------------
+
+.is_scalar_whole <- function(x, tol = .Machine$double.eps^0.5) {
+    if (length(x) != 1L || anyNA(x))
+        return(FALSE)
+
+    if (is.integer(x))
+        return(TRUE)
+
+    # TODO - note down why I use is.vector here (once I remember)
     if (is.vector(x, "double") && (abs(x - round(x)) < tol))
         return(TRUE)
 
@@ -749,7 +982,10 @@ assert_negative_or_na <- function(
 }
 
 # -------------------------------------------------------------------------
-#
-.stop <- function(msg, ..., .call = sys.call(-1L), .subclass = NULL) {
-    stop(errorCondition(msg, ..., class = .subclass, call = .call[1L]))
+
+.stopf <- function(msg, .call = sys.call(-1L), .subclass = NULL) {
+    class <- c(.subclass, "ympes-error")
+    call <- .call[1L]
+    err <- errorCondition(msg, class = class, call = call)
+    stop(err)
 }
